Package: alabaster.se
Title: Load and Save SummarizedExperiments from File
Version: 1.12.0
Date: 2024-10-16
Authors@R: person("Aaron", "Lun", role=c("aut", "cre"), email="infinite.monkeys.with.keyboards@gmail.com")
License: MIT + file LICENSE
Description: 
    Save SummarizedExperiments into file artifacts, and load them back into memory.
    This is a more portable alternative to serialization of such objects into RDS files.
    Each artifact is associated with metadata for further interpretation;
    downstream applications can enrich this metadata with context-specific properties.
Depends: SummarizedExperiment, alabaster.base
Imports: methods, alabaster.ranges, alabaster.matrix, BiocGenerics,
        S4Vectors, IRanges, GenomicRanges, jsonlite
Suggests: rmarkdown, knitr, testthat, BiocStyle
VignetteBuilder: knitr
RoxygenNote: 7.3.1
biocViews: DataImport, DataRepresentation
git_url: https://git.bioconductor.org/packages/alabaster.se
git_branch: RELEASE_3_23
git_last_commit: 4fe9760
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 20:33:51 UTC; biocbuild
Author: Aaron Lun [aut, cre]
Maintainer: Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>
